@@ -1,5 +1,5 @@
 /* Libart_LGPL - library of basic graphic primitives
- * Copyright (C) 1998 Raph Levien
+ * Copyright (C) 1998-1999 Raph Levien
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -35,26 +35,84 @@
 
 #define yes_OPTIMIZE_INNER
 
+/* Render an arc segment starting at (xc + x0, yc + y0) to (xc + x1,
+   yc + y1), centered at (xc, yc), and with given radius. Both x0^2 +
+   y0^2 and x1^2 + y1^2 should be equal to radius^2.
+
+   A positive value of radius means curve to the left, negative means
+   curve to the right.
+*/
+static void
+art_svp_vpath_stroke_arc (ArtVpath **p_vpath, int *pn, int *pn_max,
+			  double xc, double yc,
+			  double x0, double y0,
+			  double x1, double y1,
+			  double radius,
+			  double flatness)
+{
+  double theta;
+  double th_0, th_1;
+  int n_pts;
+  int i;
+  double aradius;
+
+  aradius = fabs (radius);
+  theta = 2 * M_SQRT2 * sqrt (flatness / aradius);
+  th_0 = atan2 (y0, x0);
+  th_1 = atan2 (y1, x1);
+  if (radius > 0)
+    {
+      /* curve to the left */
+      if (th_0 < th_1) th_0 += M_PI * 2;
+      n_pts = ceil ((th_0 - th_1) / theta);
+    }
+  else
+    {
+      /* curve to the right */
+      if (th_1 < th_0) th_1 += M_PI * 2;
+      n_pts = ceil ((th_1 - th_0) / theta);
+    }
+#ifdef VERBOSE
+  printf ("start %f %f; th_0 = %f, th_1 = %f, r = %f, theta = %f\n", x0, y0, th_0, th_1, radius, theta);
+#endif
+  art_vpath_add_point (p_vpath, pn, pn_max,
+		       ART_LINETO, xc + x0, yc + y0);
+  for (i = 1; i < n_pts; i++)
+    {
+      theta = th_0 + (th_1 - th_0) * i / n_pts;
+      art_vpath_add_point (p_vpath, pn, pn_max,
+			   ART_LINETO, xc + cos (theta) * aradius,
+			   yc + sin (theta) * aradius);
+#ifdef VERBOSE
+      printf ("mid %f %f\n", cos (theta) * radius, sin (theta) * radius);
+#endif
+    }
+  art_vpath_add_point (p_vpath, pn, pn_max,
+		       ART_LINETO, xc + x1, yc + y1);
+#ifdef VERBOSE
+  printf ("end %f %f\n", x1, y1);
+#endif
+}
+
 /* Assume that forw and rev are at point i0. Bring them to i1,
    joining with the vector i1 - i2.
 
-   It so happens that all invocations of this function maintain
-   the precondition i1 = i0 + 1, so we could decrease the number
-   of arguments by one. We haven't done that here, though.
+   This used to be true, but isn't now that the stroke_raw code is
+   filtering out (near)zero length vectors: {It so happens that all
+   invocations of this function maintain the precondition i1 = i0 + 1,
+   so we could decrease the number of arguments by one. We haven't
+   done that here, though.}
 
    forw is to the line's right and rev is to its left.
 
-   Also need to implement a "flatness" value for the round joins.
-
    Precondition: no zero-length vectors, otherwise a divide by
-   zero will happen.
-*/
+   zero will happen.  */
 static void
 render_seg (ArtVpath **p_forw, int *pn_forw, int *pn_forw_max,
 	    ArtVpath **p_rev, int *pn_rev, int *pn_rev_max,
 	    ArtVpath *vpath, int i0, int i1, int i2,
 	    ArtPathStrokeJoinType join,
-	    double line_width, double miter_limit)
+	    double line_width, double miter_limit, double flatness)
 {
   double dx0, dy0;
   double dx1, dy1;
@@ -65,6 +123,10 @@ render_seg (ArtVpath **p_forw, int *pn_forw, int *pn_forw_max,
   double scale;
   double cross;
 
+#ifdef VERBOSE
+  printf ("join style = %d\n", join);
+#endif
+
   /* The vectors of the lines from i0 to i1 and i1 to i2. */
   dx0 = vpath[i1].x - vpath[i0].x;
   dy0 = vpath[i1].y - vpath[i0].y;
@@ -180,6 +242,13 @@ render_seg (ArtVpath **p_forw, int *pn_forw, int *pn_forw_max,
 	  art_vpath_add_point (p_forw, pn_forw, pn_forw_max,
 			   ART_LINETO, vpath[i1].x - dmx, vpath[i1].y - dmy);
 	}
+      else if (join == ART_PATH_STROKE_JOIN_ROUND)
+	art_svp_vpath_stroke_arc (p_forw, pn_forw, pn_forw_max,
+				  vpath[i1].x, vpath[i1].y,
+				  -dlx0, -dly0,
+				  -dlx1, -dly1,
+				  line_width,
+				  flatness);
     }
   else
     {
@@ -233,6 +302,13 @@ render_seg (ArtVpath **p_forw, int *pn_forw, int *pn_forw_max,
 	  art_vpath_add_point (p_rev, pn_rev, pn_rev_max,
 			   ART_LINETO, vpath[i1].x + dmx, vpath[i1].y + dmy);
 	}
+      else if (join == ART_PATH_STROKE_JOIN_ROUND)
+	art_svp_vpath_stroke_arc (p_rev, pn_rev, pn_rev_max,
+				  vpath[i1].x, vpath[i1].y,
+				  dlx0, dly0,
+				  dlx1, dly1,
+				  -line_width,
+				  flatness);
 
     }
 }
@@ -241,11 +317,13 @@ render_seg (ArtVpath **p_forw, int *pn_forw, int *pn_forw_max,
 static void
 render_cap (ArtVpath **p_result, int *pn_result, int *pn_result_max,
 	    ArtVpath *vpath, int i0, int i1,
-	    ArtPathStrokeCapType cap, double line_width)
+	    ArtPathStrokeCapType cap, double line_width, double flatness)
 {
   double dx0, dy0;
   double dlx0, dly0;
   double scale;
+  int n_pts;
+  int i;
 
   dx0 = vpath[i1].x - vpath[i0].x;
   dy0 = vpath[i1].y - vpath[i0].y;
@@ -256,12 +334,48 @@ render_cap (ArtVpath **p_result, int *pn_result, int *pn_result_max,
   dlx0 = dy0 * scale;
   dly0 = -dx0 * scale;
 
-  /* butt */
-  art_vpath_add_point (p_result, pn_result, pn_result_max,
-		   ART_LINETO, vpath[i1].x - dlx0, vpath[i1].y - dly0);
-  art_vpath_add_point (p_result, pn_result, pn_result_max,
-		   ART_LINETO, vpath[i1].x + dlx0, vpath[i1].y + dly0);
-  /* todo: other cap types */
+#ifdef VERBOSE
+  printf ("cap style = %d\n", cap);
+#endif
+
+  switch (cap)
+    {
+    case ART_PATH_STROKE_CAP_BUTT:
+      art_vpath_add_point (p_result, pn_result, pn_result_max,
+			   ART_LINETO, vpath[i1].x - dlx0, vpath[i1].y - dly0);
+      art_vpath_add_point (p_result, pn_result, pn_result_max,
+			   ART_LINETO, vpath[i1].x + dlx0, vpath[i1].y + dly0);
+      break;
+    case ART_PATH_STROKE_CAP_ROUND:
+      n_pts = ceil (M_PI / (2.0 * M_SQRT2 * sqrt (flatness / line_width)));
+      art_vpath_add_point (p_result, pn_result, pn_result_max,
+			   ART_LINETO, vpath[i1].x - dlx0, vpath[i1].y - dly0);
+      for (i = 1; i < n_pts; i++)
+	{
+	  double theta, c_th, s_th;
+
+	  theta = M_PI * i / n_pts;
+	  c_th = cos (theta);
+	  s_th = sin (theta);
+	  art_vpath_add_point (p_result, pn_result, pn_result_max,
+			       ART_LINETO,
+			       vpath[i1].x - dlx0 * c_th - dly0 * s_th,
+			       vpath[i1].y - dly0 * c_th + dlx0 * s_th);
+	}
+      art_vpath_add_point (p_result, pn_result, pn_result_max,
+			   ART_LINETO, vpath[i1].x + dlx0, vpath[i1].y + dly0);
+      break;
+    case ART_PATH_STROKE_CAP_SQUARE:
+      art_vpath_add_point (p_result, pn_result, pn_result_max,
+			   ART_LINETO,
+			   vpath[i1].x - dlx0 - dly0,
+			   vpath[i1].y - dly0 + dlx0);
+      art_vpath_add_point (p_result, pn_result, pn_result_max,
+			   ART_LINETO,
+			   vpath[i1].x + dlx0 - dly0,
+			   vpath[i1].y + dly0 + dlx0);
+      break;
+    }
 }
 
 ArtVpath *
@@ -280,6 +394,9 @@ art_svp_vpath_stroke_raw (ArtVpath *vpath,
   ArtVpath *result;
   int n_result, n_result_max;
   double half_lw = 0.5 * line_width;
+  int closed;
+  int last, this, next, second;
+  double dx, dy;
 
   n_forw_max = 16;
   forw = art_new (ArtVpath, n_forw_max);
@@ -296,30 +413,59 @@ art_svp_vpath_stroke_raw (ArtVpath *vpath,
       n_forw = 0;
       n_rev = 0;
 
+      closed = (vpath[begin_idx].code == ART_MOVETO);
+
       /* we don't know what the first point joins with until we get to the
 	 last point and see if it's closed. So we start with the second
-	 line in the path. */
-      for (i = begin_idx; vpath[i + 1].code == ART_LINETO;
-	   i++)
+	 line in the path.
+
+	 Note: this is not strictly true (we now know it's closed from
+	 the opening pathcode), but why fix code that isn't broken?
+      */
+
+      this = begin_idx;
+      /* skip over identical points at the beginning of the subpath */
+      for (i = this + 1; vpath[i].code == ART_LINETO; i++)
 	{
-	  /* render the segment from [i] to [i + 1], joining with [i - 1]
-	     and [i + 2] */
-	  if (vpath[i + 2].code != ART_LINETO)
+	  dx = vpath[i].x - vpath[this].x;
+	  dy = vpath[i].y - vpath[this].y;
+	  if (dx * dx + dy * dy > EPSILON_2)
+	    break;
+	}
+      next = i;
+      second = next;
+
+      /* invariant: this doesn't coincide with next */
+      while (vpath[next].code == ART_LINETO)
+	{
+	  last = this;
+	  this = next;
+	  /* skip over identical points after the beginning of the subpath */
+	  for (i = this + 1; vpath[i].code == ART_LINETO; i++)
+	    {
+	      dx = vpath[i].x - vpath[this].x;
+	      dy = vpath[i].y - vpath[this].y;
+	      if (dx * dx + dy * dy > EPSILON_2)
+		break;
+	    }
+	  next = i;
+	  if (vpath[next].code != ART_LINETO)
 	    {
 	      /* reached end of path */
-	      /* todo: make "closed" detection conform to PostScript
+	      /* make "closed" detection conform to PostScript
 		 semantics (i.e. explicit closepath code rather than
-		 the fact that end of the path is the beginning) */
-	      if (vpath[i + 1].x == vpath[begin_idx].x &&
-		  vpath[i + 1].y == vpath[begin_idx].y)
+		 just the fact that end of the path is the beginning) */
+	      if (closed &&
+		  vpath[this].x == vpath[begin_idx].x &&
+		  vpath[this].y == vpath[begin_idx].y)
 		{
 		  int j;
 
 		  /* path is closed, render join to beginning */
 		  render_seg (&forw, &n_forw, &n_forw_max,
 			      &rev, &n_rev, &n_rev_max,
-			      vpath, i, i + 1, begin_idx + 1,
-			      join, half_lw, miter_limit);
+			      vpath, last, this, second,
+			      join, half_lw, miter_limit, flatness);
 
 #ifdef VERBOSE
 		  printf ("%% forw %d, rev %d\n", n_forw, n_rev);
@@ -350,8 +496,8 @@ art_svp_vpath_stroke_raw (ArtVpath *vpath,
 		  /* add to forw rather than result to ensure that
 		     forw has at least one point. */
 		  render_cap (&forw, &n_forw, &n_forw_max,
-			      vpath, i, i + 1,
-			      cap, half_lw);
+			      vpath, last, this,
+			      cap, half_lw, flatness);
 		  art_vpath_add_point (&result, &n_result, &n_result_max,
 				   ART_MOVETO, forw[0].x,
 				   forw[0].y);
@@ -364,8 +510,8 @@ art_svp_vpath_stroke_raw (ArtVpath *vpath,
 				     ART_LINETO, rev[j].x,
 				     rev[j].y);
 		  render_cap (&result, &n_result, &n_result_max,
-			      vpath, begin_idx + 1, begin_idx,
-			      cap, half_lw);
+			      vpath, second, begin_idx,
+			      cap, half_lw, flatness);
 		  art_vpath_add_point (&result, &n_result, &n_result_max,
 				   ART_LINETO, forw[0].x,
 				   forw[0].y);
@@ -374,10 +520,10 @@ art_svp_vpath_stroke_raw (ArtVpath *vpath,
 	  else
 	    render_seg (&forw, &n_forw, &n_forw_max,
 			&rev, &n_rev, &n_rev_max,
-			vpath, i, i + 1, i + 2,
-			join, half_lw, miter_limit);
+			vpath, last, this, next,
+			join, half_lw, miter_limit, flatness);
 	}
-      end_idx = i + 1;
+      end_idx = next;
     }
 
   art_free (forw);
