@@ -1462,7 +1462,7 @@ _gnome_config_get_translated_string_with_default (const char *path,
 
 	char *value= NULL;
 
-	language_list = gnome_i18n_get_language_list ("LC_ALL");
+	language_list = gnome_i18n_get_language_list ("LC_MESSAGES");
 
 	while (!value && language_list) {
 		const char *lang= language_list->data;
@@ -1660,6 +1660,10 @@ gnome_config_make_vector (const char *string, int *argcp, char ***argvp)
 	for (p = (char *) string; *p; ++p) {
 	        if (*p == '\\' && *(p+1)) {
 			++p;
+			if (space_seen){
+				count++;
+				space_seen = 0;
+			}
 		} else if (*p == ' ') {
 			space_seen = 1;
 		} else if (space_seen){
@@ -1688,11 +1692,12 @@ gnome_config_make_vector (const char *string, int *argcp, char ***argvp)
 
 		(*argvp)[count++] = tmp = s;
 
-		do {
+		while (*s) {
 			if (*s == '\\') 
 				s++;				
+			if (!*s) break;
 			*tmp++ = *s++;
-		} while (*s);
+		}
 		*tmp = '\0';
 
 		while (*p && *p == ' ')
@@ -1787,7 +1792,7 @@ _gnome_config_set_translated_string (const char *path, const char *value,
 	const char *lang;
 	char *tkey;
 
-	language_list = gnome_i18n_get_language_list("LC_ALL");
+	language_list = gnome_i18n_get_language_list("LC_MESSAGES");
 
 	lang= language_list ? language_list->data : NULL;
 
