@@ -352,6 +352,10 @@ g_unix_error_string (int error_num)
 char *
 g_concat_dir_and_file (const char *dir, const char *file)
 {
+	/* If the directory name is NULL, return only the filename */
+	if (!dir)
+		return g_strdup(file);
+
         /* If the directory name doesn't have a / on the end, we need
 	   to add one so we get a proper path to the file */
 	if (dir [strlen(dir) - 1] != PATH_SEP)
