@@ -250,6 +250,11 @@ gnome_app_destroy (GtkObject *object)
 	gtk_accel_group_unref (app->accel_group);
 	app->accel_group = NULL;
 
+	if (app->layout) {
+		gtk_object_unref (GTK_OBJECT (app->layout));
+		app->layout = NULL;
+	}
+
 	if (GTK_OBJECT_CLASS (parent_class)->destroy)
 		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
@@ -583,7 +588,7 @@ gnome_app_add_dock_item (GnomeApp *app,
 					    band_num,
 					    band_position,
 					    offset);
-	else {
+	else
 		gnome_dock_add_item (GNOME_DOCK(app->dock),
 				     GNOME_DOCK_ITEM( item),
 				     placement,
@@ -591,10 +596,10 @@ gnome_app_add_dock_item (GnomeApp *app,
 				     band_position,
 				     offset,
 				     FALSE);
-		gtk_signal_emit_by_name (GTK_OBJECT (app->dock),
-					 "layout_changed",
-					 (gpointer) app);
-	}
+
+	gtk_signal_emit_by_name (GTK_OBJECT (app->dock),
+				 "layout_changed",
+				 (gpointer) app);
 }
 
 /**
