@@ -21,9 +21,7 @@
 
 #include <gtk/gtk.h>
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
+BEGIN_GNOME_DECLS
 
 #define GNOME_TYPE_DRUID_PAGE			(gnome_druid_page_get_type ())
 #define GNOME_DRUID_PAGE(obj)			(GTK_CHECK_CAST ((obj), GNOME_TYPE_DRUID_PAGE, GnomeDruidPage))
@@ -58,10 +56,10 @@ gboolean gnome_druid_page_back     (GnomeDruidPage *druid_page);
 gboolean gnome_druid_page_cancel   (GnomeDruidPage *druid_page);
 void     gnome_druid_page_finish   (GnomeDruidPage *druid_page);
 
-
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
-
+END_GNOME_DECLS
 
 #endif /* __GNOME_DRUID_PAGE_H__ */
+
+
+
+
