@@ -17,6 +17,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
+
 #include <gnome.h>
 #include "gnome-druid.h"
 enum {
@@ -211,6 +213,8 @@ gnome_druid_size_request (GtkWidget *widget,
 			gtk_widget_size_request (GTK_WIDGET (child), &child_requisition);
 			temp_width = MAX (temp_width, child_requisition.width);
 			temp_height = MAX (temp_height, child_requisition.height);
+			if (GTK_WIDGET_MAPPED (child) && child != druid->current)
+				gtk_widget_unmap (child);
 		}
 	}
 	
@@ -691,13 +695,13 @@ gnome_druid_insert_page (GnomeDruid *druid,
 		gtk_widget_realize (GTK_WIDGET (page));
 
 	if (GTK_WIDGET_VISIBLE (GTK_WIDGET (druid)) && GTK_WIDGET_VISIBLE (GTK_WIDGET (page))) {
-		if (GTK_WIDGET_MAPPED (GTK_WIDGET (druid)))
-			gtk_widget_map (GTK_WIDGET (page));
-		gtk_widget_queue_resize (GTK_WIDGET (page));
+		if (GTK_WIDGET_MAPPED (GTK_WIDGET (page)))
+			gtk_widget_unmap (GTK_WIDGET (page));
+		gtk_widget_queue_resize (GTK_WIDGET (druid));
 	}
 	
-	/* if it's the first page, we want to bring it to the foreground. */
-	if (druid->children->data == (gpointer) page)
+	/* if it's the first and only page, we want to bring it to the foreground. */
+	if (druid->children->next == NULL)
 		gnome_druid_set_page (druid, page);
 }
 
