@@ -66,7 +66,7 @@ struct _GnomeMDIGenericChildClass {
 
 guint                gnome_mdi_generic_child_get_type (void);
 
-GnomeMDIGenericChild *gnome_mdi_generic_child_new     (gchar *name);
+GnomeMDIGenericChild *gnome_mdi_generic_child_new     (const gchar *name);
 
 void gnome_mdi_generic_child_set_view_creator     (GnomeMDIGenericChild *child,
 												   GnomeMDIChildViewCreator func,
