@@ -41,6 +41,8 @@
 #define STOCK_SEP '.'
 #define STOCK_SEP_STR "."
 
+#define GNOME_STOCK_BUTTON_PADDING 2
+
 static GnomePixmap *gnome_stock_pixmap(GtkWidget *window, const char *icon,
 				       const char *subtype);
 static GnomeStockPixmapEntry *lookup(const char *icon, const char *subtype,
@@ -1370,7 +1372,8 @@ gnome_pixmap_button(GtkWidget *pixmap, const char *text)
 	w = hbox;
 	hbox = gtk_hbox_new(FALSE, 0);
 	gtk_widget_show(hbox);
-	gtk_box_pack_start(GTK_BOX(w), hbox, TRUE, FALSE, 7);
+	gtk_box_pack_start(GTK_BOX(w), hbox, TRUE, FALSE,
+			   GNOME_STOCK_BUTTON_PADDING);
 
 	use_icon = gnome_config_get_bool("/Gnome/Icons/ButtonUseIcons=true");
 	use_label = gnome_config_get_bool("/Gnome/Icons/ButtonUseLabels=true");
@@ -1378,7 +1381,8 @@ gnome_pixmap_button(GtkWidget *pixmap, const char *text)
 	if ((use_label) || (!use_icon) || (!pixmap)) {
 		label = gtk_label_new(_(text));
 		gtk_widget_show(label);
-		gtk_box_pack_end(GTK_BOX(hbox), label, FALSE, FALSE, 7);
+		gtk_box_pack_end(GTK_BOX(hbox), label, FALSE, FALSE,
+				 GNOME_STOCK_BUTTON_PADDING);
 	}
 
 	if ((use_icon) && (pixmap)) {
@@ -1487,14 +1491,12 @@ GtkWidget *
 gnome_stock_menu_item(const char *type, const char *text)
 {
 	GtkWidget *hbox, *w, *menu_item;
-	int use_icons;
 
 	g_return_val_if_fail(type != NULL, NULL);
 	g_return_val_if_fail(gnome_stock_pixmap_checkfor(type, GNOME_STOCK_PIXMAP_REGULAR), NULL);
 	g_return_val_if_fail(text != NULL, NULL);
 
-	use_icons = gnome_config_get_bool("/Gnome/Icons/MenusUseIcons=true");
-	if (use_icons) {
+	if(gnome_preferences_get_menus_have_icons()) {
 		hbox = gtk_hbox_new(FALSE, 2);
 		gtk_widget_show(hbox);
 		w = gnome_stock_pixmap_widget(hbox, type);
