@@ -189,8 +189,9 @@ goad_server_list_get(void)
 	g_free (user_dir);
 	
 	/* System servers */
-	system_dir = gnome_unconditional_config_file (SERVER_LISTING_PATH);
-	load_servers_from (system_dir, newl, servinfo);
+	system_dir = gnome_config_file (SERVER_LISTING_PATH);
+	if (system_dir)
+		load_servers_from (system_dir, newl, servinfo);
 	g_free (system_dir);
 	
 	/* Now, user defined locations */
